@@ -48,6 +48,8 @@
 #endif
 
 #define BUF_SIZE 4096
+#define FILE_PREFIX    "file://"
+#define FILE_PREFIX_L  7
 
 
 gboolean
@@ -440,8 +442,8 @@ file_name_from_path (const gchar *file_name)
 char *
 dir_name_from_path (const gchar *path)
 {
-	register  gssize base;
-	register  gssize last_char;
+	register gssize base;
+	register gssize last_char;
 
 	if (path == NULL)
 		return NULL;
@@ -585,10 +587,104 @@ is_mime_type (const char* type, const char* pattern) {
 }
 
 
-G_CONST_RETURN char*
-get_mime_type (const char  *filename)
+GHashTable *static_strings = NULL;
+
+
+static const char *
+get_static_string (const char *s)
 {
-	return gnome_vfs_get_mime_type_for_name (filename);
+        const char *result;
+
+        if (s == NULL)
+                return NULL;
+
+        if (static_strings == NULL)
+                static_strings = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+        if (! g_hash_table_lookup_extended (static_strings, s, (gpointer*) &result, NULL)) {
+                result = g_strdup (s);
+                g_hash_table_insert (static_strings,
+                                     (gpointer) result,
+                                     GINT_TO_POINTER (1));
+        }
+
+        return result;
+}
+
+
+static const char *
+get_extension (const char *path)
+{
+        int         len;
+        int         p;
+        const char *ptr = path;
+
+        if (! path)
+                return NULL;
+
+        len = strlen (path);
+        if (len <= 1)
+                return NULL;
+
+        p = len - 1;
+        while ((p >= 0) && (ptr[p] != '.'))
+                p--;
+
+        if (p < 0)
+                return NULL;
+
+        return path + p;
+}
+
+
+static char*
+get_sample_name (const char *filename)
+{
+        const char *ext;
+
+        ext = get_extension (filename);
+        if (ext == NULL)
+                return NULL;
+
+        return g_strconcat ("a", get_extension (filename), NULL);
+}
+
+
+const char*
+get_file_mime_type (const char *filename,
+                    gboolean    fast_file_type)
+{
+	const char *result = NULL;
+	
+        if (filename == NULL)
+                return NULL;
+
+        if (fast_file_type) {
+                char *sample_name;
+                char *n1;
+
+                sample_name = get_sample_name (filename);
+                if (sample_name != NULL) {
+                        n1 = g_filename_to_utf8 (sample_name, -1, 0, 0, 0);
+                        if (n1 != NULL) {
+                                char *n2 = g_utf8_strdown (n1, -1);
+                                char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
+                                if (n3 != NULL)
+                                        result = gnome_vfs_mime_type_from_name_or_default (file_name_from_path (n3), NULL);
+                                g_free (n3);
+                                g_free (n2);
+                                g_free (n1);
+                        }
+                        g_free (sample_name);
+                }
+        } 
+        else {
+                if (uri_scheme_is_file (filename))
+                        filename = get_file_path_from_uri (filename);
+                result = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
+        }
+
+        return get_static_string (result);
 }
 
 
@@ -1133,6 +1229,20 @@ get_local_path_from_uri (const char *uri)
 }
 
 
+const char *
+get_file_path_from_uri (const char *uri)
+{
+	if (uri == NULL)
+		return NULL;
+	if (uri_scheme_is_file (uri))
+		return uri + FILE_PREFIX_L;
+	else if (uri[0] == '/')
+		return uri;
+	else
+		return NULL;
+}
+
+
 gboolean
 uri_has_scheme (const char *uri)
 {
@@ -1143,31 +1253,40 @@ uri_has_scheme (const char *uri)
 gboolean
 uri_is_local (const char *uri)
 {
-	GnomeVFSURI *vfs_uri;
-	gboolean     is_local;
+	return (! uri_has_scheme (uri)) || uri_scheme_is_file (uri);
+}
 
-	vfs_uri = new_uri_from_path (uri);
-	g_return_val_if_fail (vfs_uri != NULL, FALSE);
 
-	is_local = gnome_vfs_uri_is_local (vfs_uri);
-	gnome_vfs_uri_unref (vfs_uri);
+gboolean
+uri_scheme_is_file (const char *uri)
+{
+        if (uri == NULL)
+                return FALSE;
+        if (g_utf8_strlen (uri, -1) < FILE_PREFIX_L)
+                return FALSE;
+        return strncmp (uri, FILE_PREFIX, FILE_PREFIX_L) == 0;
 
-	return is_local;
 }
 
 
 const char *
 remove_host_from_uri (const char *uri)
 {
-	const char *idx;
+        const char *idx, *sep;
 
-	idx = strstr (uri, "://");
-	if (idx == NULL)
-		return uri;
-	idx = strstr (idx + 3, "/");
-	if (idx == NULL)
-		return NULL;
-	return idx;
+        if (uri == NULL)
+                return NULL;
+
+        idx = strstr (uri, "://");
+        if (idx == NULL)
+                return uri;
+        idx += 3;
+        if (*idx == '\0')
+                return "/";
+        sep = strstr (idx, "/");
+        if (sep == NULL)
+                return idx;
+        return sep;
 }
 
 
@@ -1207,7 +1326,7 @@ get_uri_from_path (const char *path)
 {
 	if (path == NULL)
 		return NULL;
-	if ((path == "") || (path[0] == '/'))
+	if ((path[0] == '\0') || (path[0] == '/'))
 		return g_strconcat ("file://", path, NULL);
 	return g_strdup (path);
 }
@@ -1223,7 +1342,7 @@ uricmp (const char *path1,
 	uri1 = get_uri_from_path (path1);
 	uri2 = get_uri_from_path (path2);
 
-	result = strcmp_null_tollerant (uri1, uri2);
+	result = strcmp_null_tolerant (uri1, uri2);
 
 	g_free (uri1);
 	g_free (uri2);
