@@ -674,6 +674,14 @@ create_command_from_filename (FrArchive  *archive,
 		return (archive->command != NULL);
 	}
 
+	if (file_extension_is (filename, ".tar.lzma")
+	    || file_extension_is (filename, ".tzma")) {
+		archive->command = fr_command_tar_new (archive->process,
+						       filename,
+						       FR_COMPRESS_PROGRAM_LZMA);
+		return (archive->command != NULL);
+	}
+
 	if (file_extension_is (filename, ".tar.lzo")
 	    || file_extension_is (filename, ".tzo")) {
 		archive->command = fr_command_tar_new (archive->process,
@@ -763,6 +771,12 @@ create_command_from_filename (FrArchive  *archive,
 			return (archive->command != NULL);
 		}
 
+		if (file_extension_is (filename, ".lzma")) {
+			archive->command = fr_command_cfile_new (archive->process, filename, FR_COMPRESS_PROGRAM_LZMA);
+			archive->is_compressed_file = TRUE;
+			return (archive->command != NULL);
+		}
+
 		if (file_extension_is (filename, ".lzo")) {
 			archive->command = fr_command_cfile_new (archive->process, filename, FR_COMPRESS_PROGRAM_LZOP);
 			archive->is_compressed_file = TRUE;
@@ -948,11 +962,6 @@ move_here (FrArchive *archive)
 	if (content_uri == NULL)
 		return;	
 
-	if (! path_is_dir (content_uri)) {
-		g_free (content_uri);
-		return;
-	}
-
 	parent = remove_level_from_path (content_uri);
 		
 	if (uricmp (parent, archive->priv->extraction_destination) == 0) {
@@ -1710,8 +1719,12 @@ convert_to_local_file_list (GList *file_list)
 	GList *scan;
 
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = scan->data;
-		local_file_list = g_list_prepend (local_file_list, get_local_path_from_uri (filename));
+		char *uri = scan->data;
+		char *local_filename;
+		
+		local_filename = get_local_path_from_uri (uri);
+		if (local_filename != NULL)
+			local_file_list = g_list_prepend (local_file_list, local_filename);
 	}
 
 	return local_file_list;
@@ -2596,6 +2609,7 @@ archive_type_has_issues_deleting_non_empty_folders (FrArchive *archive)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_BZ)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_BZ2)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_GZ)
+		|| (archive->command->file_type == FR_FILE_TYPE_TAR_LZMA)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_LZOP)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_COMPRESS));
 }
@@ -2937,6 +2951,7 @@ archive_type_has_issues_extracting_non_empty_folders (FrArchive *archive)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_BZ)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_BZ2)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_GZ)
+		|| (archive->command->file_type == FR_FILE_TYPE_TAR_LZMA)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_LZOP)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_COMPRESS));
 }
@@ -3386,6 +3401,7 @@ fr_archive_utils__get_file_name_ext (const char *filename)
 		".iso",
 		".jar",
 		".lzh",
+		".lzma",
 		".lzo",
 		".rar",
 		".rpm",
@@ -3394,6 +3410,7 @@ fr_archive_utils__get_file_name_ext (const char *filename)
 		".tar.bz",
 		".tar.bz2",
 		".tar.gz",
+		".tar.lzma",
 		".tar.lzo",
 		".tar.Z",
 		".taz",
