@@ -114,7 +114,8 @@ list__process_line (char     *line,
 	fdata = file_data_new ();
 
 	fields = split_line (line, 2);
-	fdata->size = g_ascii_strtoull (fields[1], NULL, 10);
+	if (strcmp (fields[1], "-1") != 0)
+		fdata->size = g_ascii_strtoull (fields[1], NULL, 10);
 	g_strfreev (fields);
 
 	if (fdata->size == 0)
@@ -139,7 +140,7 @@ list__process_line (char     *line,
 	if (*fdata->name == 0)
 		file_data_free (fdata);
 	else 
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
+		fr_command_add_file (comm, fdata);
 }
 
 
@@ -162,8 +163,8 @@ fr_command_cfile_list (FrCommand  *comm,
 		fr_process_add_arg (comm->process, comm->e_filename);
 		fr_process_end_command (comm->process);
 		fr_process_start (comm->process);
-
-	} else {
+	} 
+	else {
 		/* ... other compressors do not support this feature so 
 		 * simply use the archive size, suboptimal but there is no 
 		 * alternative. */
@@ -190,7 +191,7 @@ fr_command_cfile_list (FrCommand  *comm,
 		if (*fdata->name == 0)
 			file_data_free (fdata);
 		else 
-			comm->file_list = g_list_prepend (comm->file_list, fdata);
+			fr_command_add_file (comm, fdata);
 		
 		comm_cfile->error.type = FR_PROC_ERROR_NONE;
 		comm_cfile->error.status = 0;
@@ -233,6 +234,7 @@ fr_command_cfile_add (FrCommand     *comm,
 	fr_process_begin_command (comm->process, "cp");
 	fr_process_set_working_dir (comm->process, base_dir);
 	fr_process_add_arg (comm->process, "-f");
+	fr_process_add_arg (comm->process, "--");
 	fr_process_add_arg (comm->process, filename);
 	fr_process_add_arg (comm->process, temp_file);
 	fr_process_end_command (comm->process);
@@ -246,6 +248,7 @@ fr_command_cfile_add (FrCommand     *comm,
 	case FR_COMPRESS_PROGRAM_GZIP:
 		fr_process_begin_command (comm->process, "gzip");
 		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
 		break;
@@ -253,6 +256,7 @@ fr_command_cfile_add (FrCommand     *comm,
 	case FR_COMPRESS_PROGRAM_BZIP:
 		fr_process_begin_command (comm->process, "bzip");
 		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
 		break;
@@ -260,6 +264,7 @@ fr_command_cfile_add (FrCommand     *comm,
 	case FR_COMPRESS_PROGRAM_BZIP2:
 		fr_process_begin_command (comm->process, "bzip2");
 		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
 		break;
@@ -268,6 +273,15 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_begin_command (comm->process, "compress");
 		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, "--");
+		fr_process_add_arg (comm->process, filename);
+		fr_process_end_command (comm->process);
+		break;
+
+	case FR_COMPRESS_PROGRAM_LZMA:
+		fr_process_begin_command (comm->process, "lzma");
+		fr_process_set_working_dir (comm->process, temp_dir);
+		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
 		break;
@@ -277,16 +291,19 @@ fr_command_cfile_add (FrCommand     *comm,
 		fr_process_set_working_dir (comm->process, temp_dir);
 		fr_process_add_arg (comm->process, "-fU");
 		fr_process_add_arg (comm->process, "--no-stdin");
+		fr_process_add_arg (comm->process, "--");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
 		break;
 	}
 
+
       	/* copy compressed file to the dest dir */
 
 	fr_process_begin_command (comm->process, "cp");
 	fr_process_set_working_dir (comm->process, e_temp_dir);
 	fr_process_add_arg (comm->process, "-f");
+	fr_process_add_arg (comm->process, "--");
 	fr_process_add_arg (comm->process, "*");
 	fr_process_add_arg (comm->process, comm->e_filename);
 	fr_process_end_command (comm->process);
@@ -296,6 +313,7 @@ fr_command_cfile_add (FrCommand     *comm,
 	fr_process_begin_command (comm->process, "rm");
 	fr_process_set_sticky (comm->process, TRUE);
 	fr_process_add_arg (comm->process, "-rf");
+	fr_process_add_arg (comm->process, "--");
 	fr_process_add_arg (comm->process, e_temp_dir);
 	fr_process_end_command (comm->process);
 
@@ -390,6 +408,14 @@ fr_command_cfile_extract (FrCommand  *comm,
 		fr_process_end_command (comm->process);
 		break;
 
+	case FR_COMPRESS_PROGRAM_LZMA:
+		fr_process_begin_command (comm->process, "lzma");
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, "-d");
+		fr_process_add_arg (comm->process, e_temp_file);
+		fr_process_end_command (comm->process);
+		break;
+
 	case FR_COMPRESS_PROGRAM_LZOP:
 		fr_process_begin_command (comm->process, "lzop");
 		fr_process_set_working_dir (comm->process, temp_dir);
@@ -544,6 +570,11 @@ fr_command_cfile_new (FrProcess         *process,
 		return NULL;
 	}
 
+	if ((prog == FR_COMPRESS_PROGRAM_LZMA) &&
+	    (!is_program_in_path("lzma"))) {
+		return NULL;
+	}
+
 	if ((prog == FR_COMPRESS_PROGRAM_LZOP) &&
 	    (!is_program_in_path("lzop"))) {
 		return NULL;
@@ -561,6 +592,8 @@ fr_command_cfile_new (FrProcess         *process,
 		comm->file_type = FR_FILE_TYPE_BZIP2;
 	else if (prog == FR_COMPRESS_PROGRAM_COMPRESS)
 		comm->file_type = FR_FILE_TYPE_COMPRESS;
+	else if (prog == FR_COMPRESS_PROGRAM_LZMA)
+		comm->file_type = FR_FILE_TYPE_LZMA;
 	else if (prog == FR_COMPRESS_PROGRAM_LZOP)
 		comm->file_type = FR_FILE_TYPE_LZOP;
 
