@@ -25,7 +25,6 @@
 
 
 #include <config.h>
-#include <gnome.h>
 #include "actions.h"
 #include "fr-stock.h"
 
@@ -36,7 +35,7 @@ static GtkActionEntry action_entries[] = {
 	{ "ViewMenu", NULL, N_("_View") },
 	{ "HelpMenu", NULL, N_("_Help") },
 	{ "ArrangeFilesMenu", NULL, N_("_Arrange Files") },
-	{ "OpenRecentMenu", NULL, N_("Open R_ecent") },
+	{ "OpenRecentMenu", NULL, N_("Open _Recent") },
 
 	{ "About", GTK_STOCK_ABOUT,
 	  NULL, NULL,
@@ -84,7 +83,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Rename the selection"),
 	  G_CALLBACK (activate_action_rename) },	   
 	{ "Delete", GTK_STOCK_DELETE,
-	  NULL, NULL,
+	  NULL, "Delete",
 	  N_("Delete the selection from the archive"),
 	  G_CALLBACK (activate_action_delete) },
 	  
@@ -114,7 +113,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Deselect all files"),
 	  G_CALLBACK (activate_action_deselect_all) },
 	{ "Extract", FR_STOCK_EXTRACT,
-	  N_("_Extract..."), NULL,
+	  N_("_Extract..."), "<control>E",
 	  N_("Extract files from the archive"),
 	  G_CALLBACK (activate_action_extract) },
 	{ "ExtractFolderFromSidebar", FR_STOCK_EXTRACT,
@@ -142,7 +141,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Open archive"),
 	  G_CALLBACK (activate_action_open) },
 	{ "OpenSelection", NULL,
-	  N_("Op_en With..."), NULL,
+	  N_("_Open With..."), NULL,
 	  N_("Open selected files with an application"),
 	  G_CALLBACK (activate_action_open_with) },
 	{ "Password", NULL,
@@ -150,7 +149,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Specify a password for this archive"),
 	  G_CALLBACK (activate_action_password) },
 	{ "Properties", GTK_STOCK_PROPERTIES,
-	  NULL, NULL,
+	  NULL, "<alt>Return",
 	  N_("Show archive properties"),
 	  G_CALLBACK (activate_action_properties) },
 	{ "Reload", GTK_STOCK_REFRESH,
@@ -162,7 +161,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Save the current archive with a different name"),
 	  G_CALLBACK (activate_action_save_as) },
 	{ "SelectAll", NULL,
-	  N_("Select _All"), "<control>A",
+	  N_("_Select All"), "<control>A",
 	  N_("Select all files"),
 	  G_CALLBACK (activate_action_select_all) },
 	{ "Stop", GTK_STOCK_STOP,
@@ -316,7 +315,6 @@ static const gchar *ui_info =
 "    <separator/>"
 "    <toolitem action='AddFiles_Toolbar'/>"
 "    <toolitem action='AddFolder_Toolbar'/>"
-"    <toolitem action='ViewSelection_Toolbar'/>"
 "    <separator/>"
 "    <toolitem action='Stop'/>"
 "  </toolbar>"
